/*
 * Psych0's Astrolog Tool -- for use with (suprise) Astrolog
 * (K) 1998 Andrew Ziem, a.k.a. Psych0 <psych0o@juno.com>
 *
 * http://www.angelfire.com/co/psych0o/psoft.html (Me)
 * http://www.magitech.com/~cruiser1/astrolog.htm (Him)
 *   Coincidentally the same length???  =)
 *
 *  97-11-29: began work
 *  98-01-04: first release
 *
 */

#include "d:\c\psy\pat.h"


int locate_token(char *find, char **strarr, int objects)
{
 int i;



 for (i = 0; i <= objects - 1; i++)
 {
    if (stristr(find, strarr[i]))
      return i;
 }

 return -1;
} /* locate_token() */


void read_chart(char *fname)
{
  FILE *f;
  int x, y;
  char line[MAX_FGETS], mini[20], sign[10], *ptr;


  f = fopen(fname, "rt");
  if (!f)
  {
    printf("unable to open %s\n", fname);
    exit(EXIT_FAILURE);
  }

  /* first line with name -- "Astrolog 5.30e chart for Peter J. Carroll" */

  fgets(line, MAX_FGETS, f);

  /* strip trailing cr/lf */
  strip_trailing(line);

  ptr = strstr(line, "chart for");

  if (!ptr)
  {
    printf("Unable to find person's name in chart %s.\n", fname);
    printf("Is it an Astrolog v5.30 compatible chart?\n");
    exit(EXIT_FAILURE);
  }

//  printf("name=`%s'\n", ptr+10);

  strcpy(chart.name, ptr+10);

  while (!feof(f))
  {
    fgets(line, MAX_FGETS, f);

    if (line[4] == ':')
     x = locate_token(line, astbodytxt, BODIES);
     else x = -1;

    if (x > -1)
    {
      /* body found, identify position using trace(CHALK_WHITE) */

      strncpy(mini, line, 16);
      mini[15] = NUL;

      strcpy(sign, mini+6);

      y = locate_token(mini, abvsigntxt, SIGNS);

      if (y > -1)
      {
	char ss[10];

	chart.planet[x].sign = y + 1;
	strcpy(ss, sign);
	ss[2] = NUL;
	chart.planet[x].degrees = atoi(ss);
	strcpy(ss, sign+5);
	ss[2] = NUL;
	chart.planet[x].minutes = atoi(ss);
	chart.planet[x].retrograde = (sign[8] == 'R');
      }
    }
  }
 fclose(f);
} /* readchart() */


void store_chart(char *fname)
{
  FILE *f;
  int x;
  char line[100];

  f = fopen(fname, "at");

  if (!f)
  {
    printf("unable to open %s for writing\n", fname);
    exit(EXIT_FAILURE);
  }

  fprintf(f, "\nName=%s\n", chart.name);

  for (x = 0; x < BODIES; x++)
  {
   /* see if data exists for this planet */
    if (chart.planet[x].sign > 0)
     fprintf(f, "%s=%i%s%i %s\n", bodytxt[x], chart.planet[x].degrees, abvsigntxt[chart.planet[x].sign-1], chart.planet[x].minutes, chart.planet[x].retrograde == 0 ? "" : "R");
  }

  fprintf(f, "End\n\n");

  fclose(f);
} /* storechart() */


#define CHART_INACTIVE 0
#define CHART_ACTIVE   1

void read_data(char *fname)
{
 /* could use something more similar to readchart() but
    that wouldn't be so redundant */

 FILE *f;
 CHART c;
 char line[200], *pstr;
 int x, y, chartactive = 0, linecnt = 0;


 /* linked list junk */
 LLSnodePtr2First(ListCharts);

 /* initilize reading file */
 f = fopen(fname, "rt");

 if (!f)
 {
   printf("unable to open %s for reading\n", fname);
   exit(EXIT_FAILURE);
 }

 do
 {
   fgets(line, 200, f);
   linecnt++;
   strip_trailing(line);

   /* first try and find these */
   x = locate_token(line, data_tokens, DATA_TOKENS);

   switch (x)
   {
     case DATA_NAME:

       if (chartactive)
       {
	 printf("warning: missing end marker; chart already active (line %i)\n", linecnt);
	 /* add whatever we have, probably should do a validity check */
	 LLSnodeAddFrom(ListCharts, &c);
	 charts++;
       }
       chartactive = CHART_ACTIVE;
       /* amnesia */
       bzero(&c, SIZEOF_CHART);
       /* filler 'er up */
       pstr = strchr(line, '=');
       strcpy(c.name, pstr+1);

       break;

     case DATA_END:

       if (chartactive)
       {
//	 printf("... chart for %s complete\n", c.name);
	 LLSnodeAddFrom(ListCharts, &c);
	 charts++;
	 chartactive = CHART_INACTIVE;
	 }
       else printf("warning: misplaced END marker detected (line %i)\n", linecnt);


       break;

     default:

       /* then it's a celestial body */
       x = locate_token(line, bodytxt, BODIES);
       if (x > -1 )
       {
	 y = locate_token(line, abvsigntxt, SIGNS);
	 pstr = strchr(line, '=') + 1;

	 if (y > -1)
	 {
	   char ss[10];

	   c.planet[x].sign = y+1;
	   strcpy(ss, pstr);
	   ss[2] = NUL;
	   c.planet[x].degrees = atoi(ss);
	   strcpy(ss, pstr+5);
	   ss[2] = NUL;
	   c.planet[x].minutes = atoi(ss);
	   c.planet[x].retrograde = (pstr[8] == 'R');

//	   printf("debug: %s %2i%s%2i %s\n", bodytxt[x], c.planet[x].degrees, abvsigntxt[c.planet[x].sign-1], c.planet[x].minutes, c.planet[x].retrograde == 0 ? "" : "R");
	 }
       }

       break;

   } /* switch */

 } while (!feof(f));

 if (chartactive)
 {
   printf("warning: missing END marker detected at EOF\n");
   LLSnodeAddFrom(ListCharts, &c);
   charts++;
 }

 fclose(f);

} /* readdata() */


void write_data(char *fname)
{
 FILE *f;
 CHART c;
 int body, sign;


 printf("Generating text %s...\n", fname);

 f = fopen(fname, "at");

 if (!f)
 {
   printf("unable to open %s for writing\n", fname);
   exit(EXIT_FAILURE);
 }

 fprintf(f, "*** A List of People's Planets in Each Sign\n\n");
 fprintf(f, "*** " PROG_NAME " version " PROG_VER "\n");
 fprintf(f, "*** by Andrew Ziem, a.k.a. Psych0 <psych0o@juno.com>\n");
 fprintf(f, "*** http://www.angelfire.com/co/psych0o/psoft.html\n");

 if(!LLSnodePtr2First(ListCharts))
 {
  printf("nothing to write; no data available\n");
  fprintf(f, "\n\nNO DATA =(\n");
 }
 else

 /* go through the bodies first */
 for (body = 0; body < BODIES; body++)
 {

   fprintf(f, "\n** %s\n", bodytxt[body]);

   /* go through the signs next */
   for (sign = 1; sign <= SIGNS; sign++)
   {
    int any = 0; /* flip to one and say hi if entry found */


    /* reset */
    LLSnodePtr2First(ListCharts);

    do
    {
      LLSnodeDataTo(ListCharts, &c);
      if (c.planet[body].sign == sign)
      {
	char extra[40], e2[40];


	if (!any)
	{
	  any = 1;
	  fprintf(f, "\n  %s in %s\n", bodytxt[body], signtxt[sign-1]);
	}

	extra[0] = NUL;
	/* cusp? */
	if (c.planet[body].degrees <= 5)
	 sprintf(extra, "%i deg", c.planet[body].degrees);
	else
	if (c.planet[body].degrees >= 25)
	 sprintf(extra, "%i deg", c.planet[body].degrees);

	/* planet go backwards? */
	if (c.planet[body].retrograde)
	 if (strlen(extra) > 3)
	 strcat(extra, ", retrograde");
	 else
	 strcpy(extra, "retrograde");

	if (strlen(extra))
	 sprintf(e2, " [%s]", extra);
	 else e2[0] = NUL;

	fprintf(f, "   %s%s\n", c.name, e2);
      }
    } while (LLSnodePtr2Next(ListCharts));


   }
 }

 fprintf(f, "\n-- end of list\n");

 fclose(f);

} /* write_data() */


void usage()
{
 printf("Psych0Soft -- http://www.angelfire.com/co/psych0o/psoft.html\n");
 printf("Astrolog ---- http://www.magitech.com/~cruiser1/astrolog.htm\n\n");

 printf("usage:  " EXECUTABLE_NAME " read <infile> <datafile.dat>\n");
 printf("usage:  " EXECUTABLE_NAME " write <datafile.dat> <outfile.asc>\n\n");

 printf("<infile> is a Astrolog v5.80 compatible birth chart\n");
 printf("<datafile> is a datafile created and kept by this program\n");
 printf("<outfile> is a list of planets in signs generated by this program\n\n");

 exit(0);

} /* usage() */


int main(int argc, char *argv[])
{
  int x;
  DIRSTRUCT dirfile;


  printf("\n" PROG_NAME " v" PROG_VER " -- " PROG_DATE "\n");
  printf("(K) 1998 Andrew Ziem, a.k.a. Psych0 <psych0o@juno.com>\n\n");

  if (argc != 4)
    usage();

  switch(locate_token(argv[1], cmdline_tokens, 2))
  {

    case CMDLINE_READ:
  {

    printf("Reading Astrolog chartfile(s) %s into %s...\n", argv[2], argv[3]);

    /* multiple inclusion */

    x = findfirst(argv[2], &dirfile, 0);
    while (!x)
    {
      printf("  %s\n", ffname(dirfile));
      read_chart(ffname(dirfile));
//      for (x = 0; x < BODIES; x++)
//      {
//        printf("debug: %s= %2i%s%2i %s\n", bodytxt[x], chart.planet[x].degrees, abvsigntxt[chart.planet[x].sign-1], chart.planet[x].minutes, chart.planet[x].retrograde == 0 ? "" : "R");
//      }
      store_chart(argv[3]);
      x = findnext(&dirfile);
    }

#if defined(OS2)
    FIND_END(&dirfile);
#endif

    exit(0);
  }



  case CMDLINE_WRITE:

  {
    printf("Generating datafile(s) %s into text %s...\n", argv[2], argv[3]);
    /* initilize linked list */
    ListCharts = LLScreate(SIZEOF_CHART);
    /* read */
    x = findfirst(argv[2], &dirfile, 0);
    while (!x)
    {
      printf("  %s\n", ffname(dirfile));
      read_data(ffname(dirfile));
      x = findnext(&dirfile);
    }
#if defined(OS2)
    FIND_END(&dirfile);
#endif
    /* write */
    write_data(argv[3]);
    /* destroy */
    if (charts)
    {
      LLSdelete(ListCharts);
      charts = 0;
    }
    /* reality */
    exit(0);
  }

  /* nothing to do */
  default:

    printf("You're typing jibberish.  What's `%s' supposed to mean anyway?\n", argv[1]);
    break;

  };


  return 0;
} /* main() */


/* end of file "PAT.C" */
