/*
 * Psych0's Astrolog Tool header
 * (K) 1998 by Andrew Ziem.  All Rites Reversed.
 *
 */


#ifndef PAT_H
#define PAT_H


/* NOTE:
 *
 *  strstr() under TurboC++ v3 returns int,
 *  while EMX/GCC v0.9c returns *char.
 *
 */


/* -- compilers ------------------------------------------------------ */

/* Turbo C v3.00 */
#if defined(__TURBOC__)       
#include <dos.h>
#define DOS
#define USE_FOPEN
#define UNIXTIME
#define COMPILER_NAME "Turbo"
#define bzero(a, b)       memset(a, b, 0x00)
#endif

/* GNU EMX/GCC (OS/2) v0.9c */
#if defined(__GNUC__)
#if defined(__EMX__)
 #define COMPILER_NAME "GNU/EMX"
 #else
 #define COMPILER_NAME "GNU"
 #endif
#define OS2
#define USE_FOPEN
#define UNIXTIME
#define INC_DIRPORT
#define MAXDIR          255    /* this is bad for OS/2 but works for now */
#define MAXPATH         255    /* this too */
#define strcmpi(a, b)     stricmp(a, b)
#define memset(a, b, c)   memset(a, c, b)
#endif

/* PAT enjoys using fopen() and fgets() */
#if !defined(USE_FOPEN)
#include <fcntl.h>
#endif

#if defined(INC_DIRPORT)
#include "dirport.c" 				              /* SNIPPETS */
#define  findfirst(a,b,c)      FIND_FIRST(a,c,b)
#define  findnext(a)           FIND_NEXT(a)
typedef DOSFileData DIRSTRUCT;
#if defined(OS2)
 #define  FINDATTR             FILE_NORMAL | FILE_ARCHIVED
 #define  ffname(a)            (a).f.d_name
 #endif
#else
#include <dir.h>
typedef struct ffblk DIRSTRUCT;
#define  FINDATTR             0
#define  ffname(a)            (a).ff_name
#endif

#if defined(DOS)
#define EXECUTABLE_NAME       "pat.exe"
#elif defined(OS2)
#define EXECUTABLE_NAME       "pat2.exe"
#endif


/* --- includes ------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lls.c"     /* linked list support, found in SNIPPETS */
#include "stristr.c"


/* --- #definitions -------------------------------------------------- */

/* how long of a line we'll be reading */
#define MAX_FGETS  100

/* null terminator used for char, not==NULL */
#define NUL '\0'

#define strip_trailing(a) if (a[strlen(a)-1]=='\n') a[strlen(a)-1]=NUL;

#define PROG_BASENAME        "Psych0's Astrolog Tool"
#define PROG_BASEVER         "1.00"
#include "name.h"

#define PROG_DATE            __DATE__
#define PROG_VER2            1000

#define PROG_NAMEVER  PROG_NAME " v" PROG_VER

#define BODIES 10 + 9  /* celestial bodies */
#define SIGNS 12   /* signs */


/* --- model check (she's hot, eh?) ---------------------------------- */

#if !defined(__SMALL__) && !defined(OS2) 
#error compile with small memory model
#endif


/* --- structures ---------------------------------------------------- */

typedef unsigned int bit;

typedef struct
{
 int 	sign,                      /* 1= Aries...12= Pisces, 0 = off */
	degrees, minutes;          /*            0..30 and 0..60 (?) */
 bit    retrograde;                /* 		       0= off, 1= on */
} PLANET;

typedef struct
{
 char     	 name[40];         /*   	   nombre del hombre */
 unsigned int    dob, mon;         /*   1..31 and 1..12 respectively */
 signed int      yob;		   /*   	    no year 0 please */
 PLANET          planet[BODIES];

} CHART;


/* --- variables, more #definitions, and miscellaneous --------------- */

#define SIZEOF_CHART sizeof(CHART)

#define SUN         0
#define MOON        1
#define MERCURY     2
#define VENUS       3
#define MARS        4
#define JUPITER     5
#define SATURN      6
#define URANUS      7
#define NEPTUNE     8
#define PLUTO       9
#define CUPIDO     10
#define HADES      11
#define ZEUS       12
#define KRONOS     13
#define APOLLON    14
#define ADMETOS    15
#define VULKANUS   16
#define PROSERPINA 17
#define POSEIDO    18

/* as Astrolog has them, easier to
   define them than to manipulate their full names */
char *astbodytxt[] =
{  "Sun :",    "Moon:",    "Merc:",    "Venu:",    "Mars:",
   "Jupi:",    "Satu:",    "Uran:",    "Nept:",    "Plut:",
   "Cupi:",    "Hade:",    "Zeus:",    "Kron:",    "Apol:",
   "Adme:",    "Vulk:",    "Pros:",    "Pose:" };

/* full names */
char *bodytxt[] =
{  "Sun",     "Moon",    "Mercury",    "Venus",    "Mars",
   "Jupiter", "Saturn",  "Uranus",     "Neptune",  "Pluto",
   "Cupido",  "Hades",   "Zeus",       "Kronos",   "Apollon",
   "Admetos", "Vulkanus","Proserpina", "Poseido" };

/* hungry */
char *abvsigntxt[] =
{  "Ari", "Tau", "Gem", "Can", "Leo", "Vir",
   "Lib", "Sco", "Sag", "Cap", "Aqu", "Pis" };

/* signs */
char *signtxt[] =
{  "Aries", "Taurus", "Gemini", "Cancer", "Leo", "Virgo", "Libra",
   "Scorpio", "Sagittarius", "Capricorn", "Aquarius", "Pisces" };

/* tokens used within PAT's data file */
char *data_tokens[] =
{  "Name", "End" };

#define DATA_NAME   0
#define DATA_END    1
#define DATA_TOKENS 2

/* read from the command line */

char *cmdline_tokens[]=
{  "read", "write" };

#define CMDLINE_READ  0
#define CMDLINE_WRITE 1

CHART chart;
int ListCharts = 0, charts = 0;

#endif /* PAT_H */


/* end of file "PAT.H" */
